@@ -45,13 +45,33 @@ daemon_dir_handle_free (DaemonDirHandle *handle)
 	g_free (handle);
 }
 
+static CORBA_char *
+corba_string_or_null_dup (char *str)
+{
+	if (str != NULL) {
+		return CORBA_string_dup (str);
+	} else {
+		return CORBA_string_dup ("");
+	}
+}
+
+static char *
+decode_corba_string_or_null (CORBA_char *str, gboolean empty_is_null)
+{
+	if (empty_is_null && *str == 0) {
+		return NULL;
+	} else {
+		return g_strdup (str);
+	}
+}
+
 void
-_gnome_vfs_daemon_convert_from_corba_file_info (const GNOME_VFS_FileInfo *corba_info,
-						GnomeVFSFileInfo *file_info)
+gnome_vfs_daemon_convert_from_corba_file_info (const GNOME_VFS_FileInfo *corba_info,
+					       GnomeVFSFileInfo *file_info)
 {
 	g_free (file_info->name);
 	
-	file_info->name = g_strdup (corba_info->name);
+	file_info->name = decode_corba_string_or_null (corba_info->name, TRUE);
 
 	file_info->valid_fields = (GnomeVFSFileInfoFields) corba_info->valid_fields;
 	file_info->type = (GnomeVFSFileType) corba_info->type;
@@ -81,10 +101,10 @@ _gnome_vfs_daemon_convert_from_corba_file_info (const GNOME_VFS_FileInfo *corba_
 }
 
 void
-_gnome_vfs_daemon_convert_to_corba_file_info (const GnomeVFSFileInfo *file_info,
-					      GNOME_VFS_FileInfo *corba_info)
+gnome_vfs_daemon_convert_to_corba_file_info (const GnomeVFSFileInfo *file_info,
+					     GNOME_VFS_FileInfo *corba_info)
 {
-	corba_info->name = CORBA_string_dup (file_info->name);
+	corba_info->name = corba_string_or_null_dup (file_info->name);
 
 	corba_info->valid_fields = file_info->valid_fields;
 	corba_info->type = file_info->type;
@@ -103,16 +123,16 @@ _gnome_vfs_daemon_convert_to_corba_file_info (const GnomeVFSFileInfo *file_info,
 	corba_info->ctime = file_info->ctime;
 
 	if (file_info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_SYMLINK_NAME) {
-		corba_info->symlink_name = CORBA_string_dup (file_info->symlink_name);
+		corba_info->symlink_name = corba_string_or_null_dup (file_info->symlink_name);
 	} else {
-		corba_info->symlink_name = CORBA_string_dup ("");
+		corba_info->symlink_name = corba_string_or_null_dup ("");
 	}
 		
 	corba_info->mime_type = NULL;
 	if (file_info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_MIME_TYPE) {
-		corba_info->mime_type = CORBA_string_dup (file_info->mime_type);
+		corba_info->mime_type = corba_string_or_null_dup (file_info->mime_type);
 	} else {
-		corba_info->mime_type = CORBA_string_dup ("");
+		corba_info->mime_type = corba_string_or_null_dup ("");
 	}
 }
 
@@ -340,7 +360,7 @@ do_write (GnomeVFSMethod *method,
 	
 	CORBA_exception_init (&ev);
 	buf._maximum = buf._length = num_bytes;
-	buf._buffer = (char *)buffer;
+	buf._buffer = (unsigned char *)buffer;
 	buf._release = 0;
 	
 	res = GNOME_VFS_DaemonHandle_Write ((GNOME_VFS_DaemonHandle) method_handle,
@@ -397,7 +417,7 @@ do_seek (GnomeVFSMethod *method,
 static GnomeVFSResult
 do_tell (GnomeVFSMethod *method,
 	 GnomeVFSMethodHandle *method_handle,
-	 GnomeVFSFileOffset *offset_return)
+	 GnomeVFSFileSize *offset_return)
 {
 	GnomeVFSResult res;
 	CORBA_Environment ev;
@@ -613,7 +633,7 @@ do_read_directory (GnomeVFSMethod *method,
 	g_assert (dir_handle->current_pos < dir_handle->current_list->_length);
 	
 	corba_info = &dir_handle->current_list->_buffer[dir_handle->current_pos++];
-	_gnome_vfs_daemon_convert_from_corba_file_info (corba_info, file_info);
+	gnome_vfs_daemon_convert_from_corba_file_info (corba_info, file_info);
 	if (dir_handle->current_pos == dir_handle->current_list->_length) {
 		CORBA_free (dir_handle->current_list);
 		dir_handle->current_list = NULL;
@@ -634,7 +654,6 @@ do_get_file_info (GnomeVFSMethod *method,
 	GnomeVFSClient *client;
 	GnomeVFSResult res;
 	CORBA_Environment ev;
-	GNOME_VFS_DaemonHandle handle;
 	char *uri_str;
 	GnomeVFSClientCall *client_call;
 	GNOME_VFS_FileInfo *corba_info;
@@ -651,7 +670,6 @@ do_get_file_info (GnomeVFSMethod *method,
 	client_call = _gnome_vfs_client_call_get (context);
 	
 	CORBA_exception_init (&ev);
-	handle = CORBA_OBJECT_NIL;
 	res = GNOME_VFS_AsyncDaemon_GetFileInfo (daemon,
 						 uri_str,
 						 &corba_info,
@@ -670,9 +688,9 @@ do_get_file_info (GnomeVFSMethod *method,
 	}
 
 	if (res == GNOME_VFS_OK) {
-		_gnome_vfs_daemon_convert_from_corba_file_info (corba_info, file_info);
-		CORBA_free (corba_info);
+		gnome_vfs_daemon_convert_from_corba_file_info (corba_info, file_info);
 	}
+	CORBA_free (corba_info);
 	
 	CORBA_Object_release (daemon, NULL);
 					  
@@ -689,7 +707,6 @@ do_get_file_info_from_handle (GnomeVFSMethod *method,
 	GnomeVFSClient *client;
 	GnomeVFSResult res;
 	CORBA_Environment ev;
-	GNOME_VFS_DaemonHandle handle;
 	GnomeVFSClientCall *client_call;
 	GNOME_VFS_FileInfo *corba_info;
 	
@@ -697,7 +714,6 @@ do_get_file_info_from_handle (GnomeVFSMethod *method,
 	client_call = _gnome_vfs_client_call_get (context);
 	
 	CORBA_exception_init (&ev);
-	handle = CORBA_OBJECT_NIL;
 	res = GNOME_VFS_DaemonHandle_GetFileInfo ((GNOME_VFS_DaemonHandle) method_handle,
 						  &corba_info,
 						  options,
@@ -714,9 +730,9 @@ do_get_file_info_from_handle (GnomeVFSMethod *method,
 	}
 
 	if (res == GNOME_VFS_OK) {
-		_gnome_vfs_daemon_convert_from_corba_file_info (corba_info, file_info);
-		CORBA_free (corba_info);
+		gnome_vfs_daemon_convert_from_corba_file_info (corba_info, file_info);
 	}
+	CORBA_free (corba_info);
 	
 	return res;
 }
@@ -775,7 +791,6 @@ do_make_directory (GnomeVFSMethod *method,
 	GnomeVFSClient *client;
 	GnomeVFSResult res;
 	CORBA_Environment ev;
-	GNOME_VFS_DaemonHandle handle;
 	char *uri_str;
 	GnomeVFSClientCall *client_call;
 
@@ -791,7 +806,6 @@ do_make_directory (GnomeVFSMethod *method,
 	client_call = _gnome_vfs_client_call_get (context);
 	
 	CORBA_exception_init (&ev);
-	handle = CORBA_OBJECT_NIL;
 	res = GNOME_VFS_AsyncDaemon_MakeDirectory (daemon,
 						   uri_str,
 						   perm,
@@ -821,7 +835,6 @@ do_remove_directory (GnomeVFSMethod *method,
 	GnomeVFSClient *client;
 	GnomeVFSResult res;
 	CORBA_Environment ev;
-	GNOME_VFS_DaemonHandle handle;
 	char *uri_str;
 	GnomeVFSClientCall *client_call;
 
@@ -837,7 +850,6 @@ do_remove_directory (GnomeVFSMethod *method,
 	client_call = _gnome_vfs_client_call_get (context);
 	
 	CORBA_exception_init (&ev);
-	handle = CORBA_OBJECT_NIL;
 	res = GNOME_VFS_AsyncDaemon_RemoveDirectory (daemon,
 						     uri_str,
 						     BONOBO_OBJREF (client_call),
@@ -868,7 +880,6 @@ do_move (GnomeVFSMethod *method,
 	GnomeVFSClient *client;
 	GnomeVFSResult res;
 	CORBA_Environment ev;
-	GNOME_VFS_DaemonHandle handle;
 	char *old_uri_str;
 	char *new_uri_str;
 	GnomeVFSClientCall *client_call;
@@ -886,7 +897,6 @@ do_move (GnomeVFSMethod *method,
 	client_call = _gnome_vfs_client_call_get (context);
 	
 	CORBA_exception_init (&ev);
-	handle = CORBA_OBJECT_NIL;
 	res = GNOME_VFS_AsyncDaemon_Move (daemon,
 					  old_uri_str,
 					  new_uri_str,
@@ -918,7 +928,6 @@ do_unlink (GnomeVFSMethod *method,
 	GnomeVFSClient *client;
 	GnomeVFSResult res;
 	CORBA_Environment ev;
-	GNOME_VFS_DaemonHandle handle;
 	char *uri_str;
 	GnomeVFSClientCall *client_call;
 
@@ -934,7 +943,6 @@ do_unlink (GnomeVFSMethod *method,
 	client_call = _gnome_vfs_client_call_get (context);
 	
 	CORBA_exception_init (&ev);
-	handle = CORBA_OBJECT_NIL;
 	res = GNOME_VFS_AsyncDaemon_Unlink (daemon,
 					    uri_str,
 					    BONOBO_OBJREF (client_call),
@@ -965,7 +973,6 @@ do_check_same_fs (GnomeVFSMethod *method,
 	GnomeVFSClient *client;
 	GnomeVFSResult res;
 	CORBA_Environment ev;
-	GNOME_VFS_DaemonHandle handle;
 	char *source_uri_str;
 	char *target_uri_str;
 	GnomeVFSClientCall *client_call;
@@ -984,7 +991,6 @@ do_check_same_fs (GnomeVFSMethod *method,
 	client_call = _gnome_vfs_client_call_get (context);
 	
 	CORBA_exception_init (&ev);
-	handle = CORBA_OBJECT_NIL;
 	res = GNOME_VFS_AsyncDaemon_CheckSameFS (daemon,
 						 source_uri_str,
 						 target_uri_str,
@@ -1019,7 +1025,6 @@ do_set_file_info (GnomeVFSMethod *method,
 	GnomeVFSClient *client;
 	GnomeVFSResult res;
 	CORBA_Environment ev;
-	GNOME_VFS_DaemonHandle handle;
 	char *uri_str;
 	GnomeVFSClientCall *client_call;
 	GNOME_VFS_FileInfo *corba_info;
@@ -1034,12 +1039,11 @@ do_set_file_info (GnomeVFSMethod *method,
 	uri_str = gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_NONE);
 
 	corba_info = GNOME_VFS_FileInfo__alloc ();
-	_gnome_vfs_daemon_convert_to_corba_file_info (info, corba_info);
+	gnome_vfs_daemon_convert_to_corba_file_info (info, corba_info);
 	
 	client_call = _gnome_vfs_client_call_get (context);
 
 	CORBA_exception_init (&ev);
-	handle = CORBA_OBJECT_NIL;
 	res = GNOME_VFS_AsyncDaemon_SetFileInfo (daemon,
 						 uri_str,
 						 corba_info,
@@ -1067,7 +1071,6 @@ do_truncate (GnomeVFSMethod *method,
 	GnomeVFSClient *client;
 	GnomeVFSResult res;
 	CORBA_Environment ev;
-	GNOME_VFS_DaemonHandle handle;
 	char *uri_str;
 	GnomeVFSClientCall *client_call;
 
@@ -1083,7 +1086,6 @@ do_truncate (GnomeVFSMethod *method,
 	client_call = _gnome_vfs_client_call_get (context);
 	
 	CORBA_exception_init (&ev);
-	handle = CORBA_OBJECT_NIL;
 	res = GNOME_VFS_AsyncDaemon_Truncate (daemon,
 					      uri_str,
 					      where,
@@ -1118,7 +1120,6 @@ do_find_directory (GnomeVFSMethod *method,
 	GnomeVFSClient *client;
 	GnomeVFSResult res;
 	CORBA_Environment ev;
-	GNOME_VFS_DaemonHandle handle;
 	char *near_uri_str;
 	char *result_uri_str;
 	GnomeVFSClientCall *client_call;
@@ -1135,7 +1136,6 @@ do_find_directory (GnomeVFSMethod *method,
 	client_call = _gnome_vfs_client_call_get (context);
 	
 	CORBA_exception_init (&ev);
-	handle = CORBA_OBJECT_NIL;
 	res = GNOME_VFS_AsyncDaemon_FindDirectory (daemon,
 						   near_uri_str,
 						   kind,
@@ -1160,8 +1160,8 @@ do_find_directory (GnomeVFSMethod *method,
 
 	if (res == GNOME_VFS_OK) {
 		*result_uri = gnome_vfs_uri_new (result_uri_str);
-		CORBA_free (result_uri_str);
 	}
+	CORBA_free (result_uri_str);
 	
 	return res;
 }
@@ -1176,7 +1176,6 @@ do_create_symbolic_link (GnomeVFSMethod *method,
 	GnomeVFSClient *client;
 	GnomeVFSResult res;
 	CORBA_Environment ev;
-	GNOME_VFS_DaemonHandle handle;
 	char *uri_str;
 	GnomeVFSClientCall *client_call;
 
@@ -1192,7 +1191,6 @@ do_create_symbolic_link (GnomeVFSMethod *method,
 	client_call = _gnome_vfs_client_call_get (context);
 	
 	CORBA_exception_init (&ev);
-	handle = CORBA_OBJECT_NIL;
 	res = GNOME_VFS_AsyncDaemon_CreateSymbolicLink (daemon,
 							uri_str,
 							target_reference,
