@@ -34,10 +34,10 @@
 #include "gnome-vfs-mime-private.h"
 #include "gnome-vfs-mime.h"
 #include "gnome-vfs-private-utils.h"
-#include <libgnomevfs/gnome-vfs-i18n.h>
+#include "xdgmime.h"
+#include <glib/gi18n-lib.h>
 
 #include <libxml/xmlreader.h>
-#include <dirent.h>
 #include <stdio.h>
 #include <string.h>
 #include <sys/stat.h>
@@ -56,13 +56,14 @@ typedef struct {
 	char *path;
 } MimeDirectory;
 
+#if 0
 /* These ones are used to automatically reload mime info on demand */
 static time_t last_checked;
+#endif
 
 /* To initialize the module automatically */
 static gboolean gnome_vfs_mime_inited = FALSE;
 
-static GList *current_lang = NULL;
 /* we want to replace the previous key if the current key has a higher
    language level */
 
@@ -70,6 +71,8 @@ static GList *mime_directories = NULL;
 
 static GHashTable *mime_entries = NULL;
 
+G_LOCK_EXTERN (gnome_vfs_mime_mutex);
+
 static gboolean
 does_string_contain_caps (const char *string)
 {
@@ -106,7 +109,7 @@ add_data_dir (const char *dir)
 
 	directory = g_new0 (MimeDirectory, 1);
 
-	directory->path = g_build_filename (dir, "/mime/", NULL);
+	directory->path = g_build_filename (dir, "mime", NULL);
 
 	mime_directories = g_list_append (mime_directories, directory);
 }
@@ -115,52 +118,27 @@ static void
 gnome_vfs_mime_init (void)
 {
 	const char *xdg_data_home;
-	const char *xdg_data_dirs;
-	char **split_data_dirs;
-	int i;
+	const char * const *xdg_data_dirs;
 
 	/*
 	 * The hash tables that store the mime keys.
 	 */
-	current_lang = gnome_vfs_i18n_get_language_list ("LC_MESSAGES");
-
 	mime_entries = g_hash_table_new_full (g_str_hash, 
 					      g_str_equal,
 					      g_free,
 					      (GDestroyNotify)mime_entry_free);
-	
-	xdg_data_home = g_getenv ("XDG_DATA_HOME");
-	if (xdg_data_home) {
-		add_data_dir (xdg_data_home);
-	} else {
-		const char *home;
-		
-		home = g_getenv ("HOME");
-		if (home) {
-			char *guessed_xdg_home;
-			
-			guessed_xdg_home = g_build_filename (home, 
-							     "/.local/share/",
-							     NULL);
-			add_data_dir (guessed_xdg_home);
-			g_free (guessed_xdg_home);
-		}
-	}
 
-	xdg_data_dirs = g_getenv ("XDG_DATA_DIRS");
-	if (!xdg_data_dirs) {
-		xdg_data_dirs = "/usr/local/share/:/usr/share/";
-	}
-	
-	split_data_dirs = g_strsplit (xdg_data_dirs, ":", 0);
-	
-	for (i = 0; split_data_dirs[i] != NULL; i++) {
-		add_data_dir (split_data_dirs[i]);
-	}
-
-	g_strfreev (split_data_dirs);
+	xdg_data_home = g_get_user_data_dir ();
+	add_data_dir (xdg_data_home);
 
+	for (xdg_data_dirs = g_get_system_data_dirs (); 
+	     *xdg_data_dirs; 
+	     xdg_data_dirs++) {
+		add_data_dir (*xdg_data_dirs);
+	}
+#if 0
 	last_checked = time (NULL);
+#endif
 	gnome_vfs_mime_inited = TRUE;
 }
 
@@ -178,10 +156,10 @@ reload_if_needed (void)
 	if (gnome_mime_dir.force_reload || user_mime_dir.force_reload)
 		need_reload = TRUE;
 	else if (now > last_checked + 5) {
-		if (stat (gnome_mime_dir.dirname, &s) != -1 &&
+		if (g_stat (gnome_mime_dir.dirname, &s) != -1 &&
 		    s.st_mtime != gnome_mime_dir.s.st_mtime)
 			need_reload = TRUE;
-		else if (stat (user_mime_dir.dirname, &s) != -1 &&
+		else if (g_stat (user_mime_dir.dirname, &s) != -1 &&
 			 s.st_mtime != user_mime_dir.s.st_mtime)
 			need_reload = TRUE;
 	}
@@ -216,7 +194,7 @@ _gnome_vfs_mime_info_shutdown (void)
  *
  * Reload the MIME database from disk and notify any listeners
  * holding active #GnomeVFSMIMEMonitor objects.
- **/
+ */
 void
 gnome_vfs_mime_info_reload (void)
 {
@@ -233,8 +211,8 @@ gnome_vfs_mime_info_reload (void)
  * gnome_vfs_mime_freeze:
  *
  * Freezes the mime data so that you can do multiple
- * updates to the dat in one batch without needing
- * to back the files to disk or readind them
+ * updates to the data in one batch without needing
+ * to back the files to disk or reading them.
  */
 void
 gnome_vfs_mime_freeze (void)
@@ -248,8 +226,8 @@ gnome_vfs_mime_freeze (void)
  * gnome_vfs_mime_thaw:
  *
  * UnFreezes the mime data so that you can do multiple
- * updates to the dat in one batch without needing
- * to back the files to disk or readind them
+ * updates to the data in one batch without needing
+ * to back the files to disk or reading them.
  */
 void
 gnome_vfs_mime_thaw (void)
@@ -257,20 +235,26 @@ gnome_vfs_mime_thaw (void)
 	/* noop, get rid of this once all the mime editing stuff is gone */
 }
 
+/* this gives us a number of the language in the current language list,
+   the lower the number the "better" the translation */
 static int
-language_level (const char *language) 
+language_level (const char *lang)
 {
-        int i;
-        GList *li;
- 
-        if (language == NULL)
-                return 0;
- 
-        for (i = 1, li = current_lang; li != NULL; i++, li = g_list_next (li)) {                if (strcmp ((const char *) li->data, language) == 0)
-                        return i;
-        }
- 
-        return -1;
+	const char * const *lang_list;
+	int i;
+
+	if (lang == NULL)
+		lang = "C";
+
+	/* The returned list is sorted from most desirable to least
+            desirable and always contains the default locale "C". */
+	lang_list = g_get_language_names();
+
+	for (i = 0; lang_list[i]; i++)
+		if (strcmp (lang_list[i], lang) == 0)
+			return i;
+
+	return -1;
 }
 
 static int
@@ -308,7 +292,7 @@ handle_simple_string (xmlTextReaderPtr reader)
 			if (text != NULL) {
 				g_free (text);
 			}
-			text = g_strdup (xmlTextReaderConstValue (reader));
+			text = g_strdup ((char *)xmlTextReaderConstValue (reader));
 		}
 
 		ret = read_next (reader);
@@ -323,9 +307,9 @@ handle_attribute (xmlTextReaderPtr  reader,
 	xmlChar *xml_text = NULL;
 	char *text = NULL;
 
-	xml_text = xmlTextReaderGetAttribute (reader, attribute);
+	xml_text = xmlTextReaderGetAttribute (reader, (guchar *)attribute);
 	if (xml_text != NULL) {
-		text = g_strdup (xml_text);
+		text = g_strdup ((char *)xml_text);
 		xmlFree (xml_text);
 	}
 	return text;
@@ -336,13 +320,10 @@ handle_mime_info (const char *filename, xmlTextReaderPtr reader)
 {
 	MimeEntry *entry;
 	int ret;
-	int depth;
-	int previous_lang_level = -1;
+	int previous_lang_level = INT_MAX;
 	
 	entry = g_new0 (MimeEntry, 1);
 
-	depth = xmlTextReaderDepth (reader);
-
 	ret = xmlTextReaderRead (reader);
 	while (ret == 1) {
 		xmlReaderTypes type;
@@ -350,17 +331,18 @@ handle_mime_info (const char *filename, xmlTextReaderPtr reader)
 		
 		if (type == XML_READER_TYPE_ELEMENT) {
 			const char *name;
-			name = xmlTextReaderConstName (reader);
+			name = (char *)xmlTextReaderConstName (reader);
 			
 			if (!strcmp (name, "comment")) {
 				const char *lang;
 				int lang_level;
 				
-				lang = xmlTextReaderConstXmlLang (reader);
+				lang = (char *)xmlTextReaderConstXmlLang (reader);
 				
 				lang_level = language_level (lang);
 				
-				if (lang_level > previous_lang_level) {
+				if (lang_level != -1 &&
+				    lang_level < previous_lang_level) {
 					char *comment;
 					comment = handle_simple_string (reader);
 					g_free (entry->description);
@@ -486,21 +468,26 @@ get_mime_entry_path (const char *mime_type)
 static MimeEntry *
 get_entry (const char *mime_type)
 {
+	const char *umime;
 	MimeEntry *entry;
 	char *path;
-	
-	entry = g_hash_table_lookup (mime_entries, mime_type);
+
+	G_LOCK (gnome_vfs_mime_mutex);
+	umime =	xdg_mime_unalias_mime_type (mime_type);
+	G_UNLOCK (gnome_vfs_mime_mutex);
+
+	entry = g_hash_table_lookup (mime_entries, umime);
 	
 	if (entry) {
 		return entry;
 	}
 	
-	path = get_mime_entry_path (mime_type);
+	path = get_mime_entry_path (umime);
 
 	if (path) {
-		entry = load_mime_entry (mime_type, path);
+		entry = load_mime_entry (umime, path);
 		g_hash_table_insert (mime_entries, 
-				     g_strdup (mime_type), 
+				     g_strdup (umime), 
 				     entry);
 		g_free (path);
 		return entry;
@@ -510,19 +497,17 @@ get_entry (const char *mime_type)
 }
 
 /**
- * gnome_vfs_mime_set_value
+ * gnome_vfs_mime_set_value:
  * @mime_type: a mime type.
  * @key: a key to store the value in.
- * @value: the value to store in the key.
+ * @value: the value to store in the @key.
  *
- * This function is going to set the value
- * associated to the key and it will save it
+ * This function will set the @value for the @key and it will save it
  * to the user' file if necessary.
- * You should not free the key/values passed to
- * this function. They are used internally.
- *
- * Returns: GNOME_VFS_OK if the operation succeeded, otherwise an error code
+ * You should not free the @key/@value passed to this function.
+ * They are used internally.
  *
+ * Returns: %GNOME_VFS_OK if the operation succeeded, otherwise an error code.
  */
 GnomeVFSResult
 gnome_vfs_mime_set_value (const char *mime_type, const char *key, const char *value)
@@ -534,13 +519,13 @@ gnome_vfs_mime_set_value (const char *mime_type, const char *key, const char *va
 /**
  * gnome_vfs_mime_get_value:
  * @mime_type: a mime type.
- * @key: A key to lookup for the given mime-type
+ * @key: a key to lookup for the given @mime_type.
  *
  * This function retrieves the value associated with @key in
- * the given GnomeMimeContext.  The string is private, you
+ * the given #GnomeMimeContext.  The string is private, you
  * should not free the result.
  *
- * Returns: GNOME_VFS_OK if the operation succeeded, otherwise an error code
+ * Returns: %GNOME_VFS_OK if the operation succeeded, otherwise an error code.
  */
 const char *
 gnome_vfs_mime_get_value (const char *mime_type, const char *key)
@@ -550,12 +535,6 @@ gnome_vfs_mime_get_value (const char *mime_type, const char *key)
 	if (!gnome_vfs_mime_inited)
 		gnome_vfs_mime_init ();
 
-	/* TODO: We really should handle aliases here.
-	   For now, special case dirs */
-	if (strcmp (mime_type, "x-directory/normal") == 0)
-	  mime_type = "inode/directory";
-	  
-	
 	entry = get_entry (mime_type);
 
 	if (!entry) {
@@ -581,9 +560,9 @@ gnome_vfs_mime_get_value (const char *mime_type, const char *key)
  * gnome_vfs_mime_type_is_known:
  * @mime_type: a mime type.
  *
- * This function returns TRUE if @mime_type is in the MIME database at all.
+ * This function returns %TRUE if @mime_type is in the MIME database at all.
  *
- * Returns: TRUE if anything is known about @mime_type, otherwise FALSE
+ * Returns: %TRUE if anything is known about @mime_type, otherwise %FALSE.
  */
 gboolean
 gnome_vfs_mime_type_is_known (const char *mime_type)
@@ -604,20 +583,17 @@ gnome_vfs_mime_type_is_known (const char *mime_type)
 
 	entry = get_entry (mime_type);
 	
-	/* TODO: Should look for aliases too, which needs
-	   a alias -> mimetype mapping */
-
 	return entry != NULL;
 }
 
 /**
  * gnome_vfs_mime_get_extensions_list:
- * @mime_type: type to get the extensions of
+ * @mime_type: mime type to get the extensions of.
  *
  * Get the file extensions associated with mime type @mime_type.
  *
- * Return value: a GList of char *s
- **/
+ * Return value: a #GList of char *s.
+ */
 GList *
 gnome_vfs_mime_get_extensions_list (const char *mime_type)
 {
@@ -627,11 +603,11 @@ gnome_vfs_mime_get_extensions_list (const char *mime_type)
 
 /**
  * gnome_vfs_mime_extensions_list_free:
- * @list: the extensions list
+ * @list: the extensions list.
  *
- * Call this function on the list returned by gnome_vfs_mime_extensions
- * to free the list and all of its elements.
- **/
+ * Call this function on the list returned by gnome_vfs_mime_get_extensions_list()
+ * to free the @list and all of its elements.
+ */
 void
 gnome_vfs_mime_extensions_list_free (GList *list)
 {
